/*
 * Copyright (c) 2007, WSO2 Inc. (http://www.wso2.org) All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.registry.app;

import org.apache.abdera.protocol.server.impl.AbstractSingletonProviderManager;
import org.apache.abdera.protocol.server.impl.DefaultServiceContext;
import org.apache.abdera.protocol.server.servlet.AbderaServlet;
import org.apache.abdera.protocol.server.ServiceContext;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.methods.GetMethod;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;
import org.wso2.registry.jdbc.JDBCRegistry;
import org.wso2.registry.jdbc.InMemoryJDBCRegistry;
import org.wso2.registry.jdbc.realm.RegistryRealm;
import org.wso2.registry.jdbc.realm.InMemoryRegistryRealm;
import org.wso2.registry.RegistryConstants;
import org.wso2.registry.RegistryException;

import javax.servlet.ServletException;

import junit.framework.TestCase;

/**
 * These tests use a raw HTTP client in order to exercise the APP implementation of our Registry
 * server.  We do this to make sure that the protocol exactly matches what we expect; using the
 * RemoteRegistry (as the JettyBasedServerTest does) is a "black box" test which only proves the
 * API works.  This test confirms wire-level behavior.
 *
 * TODO: WORK IN PROGRESS (Glen)
 */
public class APPTests extends TestCase implements RegistryConstants {
    DefaultServiceContext abderaServiceContext;

    public void setUp() throws Exception {
        // Set up Jetty server

        abderaServiceContext = new RegistryAPPServiceContext();
        RegistryAPPTargetResolver resolver = new RegistryAPPTargetResolver();
        abderaServiceContext.setTargetResolver(resolver);
        AbstractSingletonProviderManager pm = new RegistryAtomProviderManager();
        abderaServiceContext.setProviderManager(pm);

        Server server = new Server(8081);
        Context root = new Context(server, "/wso2registry", Context.NO_SESSIONS);
        root.addServlet(new ServletHolder(new AbderaServlet() {

            @Override
            protected ServiceContext createServiceContext() {
                abderaServiceContext
                        .setDefaultProviderManager(RegistryAtomProviderManager.class.getName());
                abderaServiceContext
                        .setDefaultTargetResolver(RegistryAPPTargetResolver.class.getName());
                abderaServiceContext.init(getAbdera(), getProperties(getServletConfig()));
                JDBCRegistry jdbcRegistry;
                try {
                    RegistryRealm realm = new InMemoryRegistryRealm();
                    jdbcRegistry = new InMemoryJDBCRegistry(realm);
                    //Realm registryRealm =
                    //        RegistryRealmFactory.createInMemoryRegistryRealm();
                    //jdbcRegistry = new JDBCRegistry();
                    //jdbcRegistry.init(registryRealm);
                    System.getProperties().put(RegistryConstants.REGISTRY, jdbcRegistry);
                } catch (RegistryException e) {
                    e.printStackTrace();
                }
                return abderaServiceContext;
            }

            public void init() throws ServletException {
                super.init();
                System.out.println("done");
            }
        }), "/*");
        server.start();
    }

    String PORT = "8081";
    String BASEURL = "http://localhost:" + PORT + "/wso2registry/atom/";

    public String buildURL(String path) {
        return BASEURL + path;
    }

    /**
     * Run through a series of actions against a clean registry, and confirm the results
     * match our expectations.
     *
     * @throws Exception if there's a problem
     */
    public void testProtocol() throws Exception {
        // Note - every URL below is rooted at our Registry's atom root

        // GET the root - empty dir
        HttpClient client = new HttpClient();
        HttpMethod method = new GetMethod(buildURL(""));
        int responseCode = client.executeMethod(method);
        assertEquals(200, responseCode);

        byte [] response = method.getResponseBody();

        // GET /d1 - 404
        method = new GetMethod(buildURL("d1"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode); // Nothing there yet!

        // GET /foo;tags - 404
        // Make sure metadata indicator doesn't change error code
        method = new GetMethod(buildURL("foo" + URL_SEPARATOR + "tags"));
        responseCode = client.executeMethod(method);
        assertEquals(404, responseCode);

        // POST / with d1 content
        PostMethod writeMethod = new PostMethod(buildURL(""));
        String atomContent =
                "<feed></feed>";
        StringRequestEntity myRequest = new StringRequestEntity(atomContent);
        writeMethod.setRequestEntity(myRequest);
        client.executeMethod(writeMethod);
        
        // GET /d1 - empty dir
        // GET /d1;tags - empty tags (or 404?)

        // POST / with no slug - text file
        // GET /<newname> -

        // POST /d1 with bad slug "&test?"
        // Should return /d1/test (sanitized name)
        // GET /d1/test - content

        // POST /d1 with existing slug "test"
        // Should return /d1/test2 (edited name)
        // GET /d1/test2 - content

        // Tags

        // GET /d1/test;tags - empty tags (or 404?)
        // POST /d1/test;tags tag=blue
        // GET /d1/test;tags - "blue"

        // Ratings

        // Comments
    }
}
